<?php

namespace Chemical\Subscriber\Policies;

use Litepie\User\Contracts\UserPolicy;
use Chemical\Subscriber\Models\Subscriber;

class SubscriberPolicy
{

    /**
     * Determine if the given user can view the subscriber.
     *
     * @param UserPolicy $user
     * @param Subscriber $subscriber
     *
     * @return bool
     */
    public function view(UserPolicy $user, Subscriber $subscriber)
    {
        if ($user->canDo('subscriber.subscriber.view') && $user->isAdmin()) {
            return true;
        }

        return $subscriber->user_id == user_id() && $subscriber->user_type == user_type();
    }

    /**
     * Determine if the given user can create a subscriber.
     *
     * @param UserPolicy $user
     * @param Subscriber $subscriber
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('subscriber.subscriber.create');
    }

    /**
     * Determine if the given user can update the given subscriber.
     *
     * @param UserPolicy $user
     * @param Subscriber $subscriber
     *
     * @return bool
     */
    public function update(UserPolicy $user, Subscriber $subscriber)
    {
        if ($user->canDo('subscriber.subscriber.edit') && $user->isAdmin()) {
            return true;
        }

        return $subscriber->user_id == user_id() && $subscriber->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given subscriber.
     *
     * @param UserPolicy $user
     * @param Subscriber $subscriber
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Subscriber $subscriber)
    {
        return $subscriber->user_id == user_id() && $subscriber->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given subscriber.
     *
     * @param UserPolicy $user
     * @param Subscriber $subscriber
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Subscriber $subscriber)
    {
        if ($user->canDo('subscriber.subscriber.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given subscriber.
     *
     * @param UserPolicy $user
     * @param Subscriber $subscriber
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Subscriber $subscriber)
    {
        if ($user->canDo('subscriber.subscriber.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
